package bus

import (
	"context"
	"errors"
	"fmt"
	"pay/db/model"
	"pay/log"
	"pay/utils"

	"github.com/nats-io/nats.go"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"infish.cn/comm"
	"infish.cn/comm/pay"
)

type SinglePay struct{}

// 创建二维码
func (s *SinglePay) CreateQr() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleQrApi)

	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleQrApi,
		Entity:  func() interface{} { return &pay.OrderMsg{} },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			m := entity.(*pay.OrderMsg)
			pay := PayMod(int(*m.PayMod))
			ctx := context.Background()
			res, err := pay.Pay(ctx, m)
			return res, err
		},
	}
}

// 关闭订单应答
func (s *SinglePay) CloseOrder() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleCloseOrderApi)

	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleCloseOrderApi,
		Entity:  func() interface{} { return &pay.OrderMsg{} },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			m := entity.(*pay.OrderMsg)
			pay := PayMod(int(*m.PayMod))
			ctx := context.Background()
			flag, err := pay.Close(ctx, m)
			return flag, err
		},
	}
}

// 退款
func (s *SinglePay) Refund() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleRefundApi)
	refundId := primitive.ObjectID{}
	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleRefundApi,
		Entity:  func() interface{} { return &refundId },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			refundId := entity.(*primitive.ObjectID)
			cxt := context.Background()
			refund, err := getPayRefund(cxt, map[string]interface{}{"_id": refundId})
			if err != nil {
				return false, errors.New("该退款申请信息错误")
			}
			// ???适配缺失退款理由
			if refund.Reason == "" {
				refund.Reason = "默认退款"
			}
			payi := PayMod(int(*refund.PayMod))
			ctx := context.Background()
			flag, err := payi.ReFundPay(ctx, refund)
			// 根据退款结果更新退款状态
			var status int32 = 0
			if flag.(bool) {
				status = int32(pay.REFUND_SUCCESS)
			} else {
				// 退款异常
				status = int32(pay.REFUND_ABNORMAL)
			}
			_, resErr := updateRefund(ctx, &pay.ReFund{Status: &status})
			if resErr != nil {
				log.Error(err)
				fmt.Println("更新退款状态失败")
			}
			return flag, err
		},
	}
}

// 退款查询
func (s *SinglePay) RefundQuery() *comm.NatsMsgReplyer {
	utils.ConsoleFormat(pay.PaySingleRefundQueryApi)

	return &comm.NatsMsgReplyer{
		Subject: pay.PaySingleRefundQueryApi,
		Entity:  func() interface{} { return &model.ReFund{} },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			m := entity.(*model.ReFund)
			pay := PayMod(int(*m.PayMode))
			ctx := context.Background()
			status, err := pay.ReFundQuery(ctx, m)
			return status, err
		},
	}
}
