package bus

import (
	"context"
	"pay/db"
	"pay/db/repo"
	"pay/utils"
	"time"

	"github.com/nats-io/nats.go"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"infish.cn/comm"
	"infish.cn/comm/pay"
)

// 创建订单退款申请
func PayRefundCreate() *comm.NatsMsgReplyer {
	refund := pay.ReFund{}
	utils.ConsoleFormat(pay.PayRefundCreateApi)
	return &comm.NatsMsgReplyer{
		Subject: pay.PayRefundCreateApi,
		Entity:  func() interface{} { return &refund },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			refund := entity.(*pay.ReFund)
			ctx := context.Background()
			res, err := createRefund(ctx, refund)
			return res, err
		},
	}
}

func createRefund(ctx context.Context, refund *pay.ReFund) (id primitive.ObjectID, err error) {
	_id, err := repo.RepoAddDoc(&repo.RepoSession{Ctx: ctx, Client: db.GMongoDb}, repo.CollectionRefund, &refund)
	if err != nil {
		return primitive.NilObjectID, err
	}
	id, err = primitive.ObjectIDFromHex(_id)
	return
}

// query := map[string]interface{}{"_id":"xxx"}
func PayRefundDetail() *comm.NatsMsgReplyer {
	qmap := make(map[string]interface{})
	utils.ConsoleFormat(pay.PayRefundDetailApi)
	return &comm.NatsMsgReplyer{
		Subject: pay.PayRefundDetailApi,
		Entity:  func() interface{} { return &qmap },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			query := entity.(*map[string]interface{})
			ctx := context.Background()
			res, err := getPayRefund(ctx, *query)
			return res, err
		},
	}
}

func getPayRefund(ctx context.Context, query map[string]interface{}) (refund *pay.ReFund, err error) {
	_, err = repo.RepoSeachDoc(&repo.RepoSession{Ctx: ctx, Client: db.GMongoDb}, &repo.DocSearchOptions{
		CollectName: repo.CollectionRefund,
		Query:       query,
	}, &refund)
	return
}

// 更新退款
func PayRefundUpdate() *comm.NatsMsgReplyer {
	refund := pay.ReFund{}
	utils.ConsoleFormat(pay.PayRefundUpdateApi)
	return &comm.NatsMsgReplyer{
		Subject: pay.PayRefundUpdateApi,
		Entity:  func() interface{} { return &refund },
		Cb2: func(_ *nats.Msg, entity interface{}) (interface{}, error) {
			refund := entity.(*pay.ReFund)
			ctx := context.Background()
			res, err := updateRefund(ctx, refund)
			return res, err
		},
	}
}

func updateRefund(ctx context.Context, refund *pay.ReFund) (result *mongo.UpdateResult, err error) {
	refund.UpdateTime = time.Now()
	result, err = repo.RepoUpdateSetDoc(&repo.RepoSession{Ctx: ctx, Client: db.GMongoDb}, repo.CollectionRefund, refund.Id.Hex(), &refund)

	return
}
